"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.BaseTypeSets = void 0;
exports.filterTypedFields = filterTypedFields;
exports.triggerOptions = exports.systemOptions = exports.scopeOptions = exports.nodesOptions = void 0;
exports.useCollectionFieldOptions = useCollectionFieldOptions;
exports.useWorkflowVariableOptions = useWorkflowVariableOptions;
function _client() {
  const data = require("@nocobase/client");
  _client = function _client() {
    return data;
  };
  return data;
}
var _FlowContext = require("./FlowContext");
var _locale = require("./locale");
var _nodes = require("./nodes");
var _triggers = require("./triggers");
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
const nodesOptions = {
  label: `{{t("Node result", { ns: "${_locale.NAMESPACE}" })}}`,
  value: '$jobsMapByNodeId',
  useOptions(options) {
    const current = (0, _nodes.useNodeContext)();
    const upstreams = (0, _nodes.useAvailableUpstreams)(current);
    const result = [];
    upstreams.forEach(node => {
      var _instruction$useVaria;
      const instruction = _nodes.instructions.get(node.type);
      const subOptions = (_instruction$useVaria = instruction.useVariables) === null || _instruction$useVaria === void 0 ? void 0 : _instruction$useVaria.call(instruction, node, options);
      if (subOptions) {
        var _node$title;
        result.push({
          key: node.id.toString(),
          value: node.id.toString(),
          label: (_node$title = node.title) !== null && _node$title !== void 0 ? _node$title : `#${node.id}`,
          children: subOptions
        });
      }
    });
    return result;
  }
};
exports.nodesOptions = nodesOptions;
const triggerOptions = {
  label: `{{t("Trigger variables", { ns: "${_locale.NAMESPACE}" })}}`,
  value: '$context',
  useOptions(options) {
    var _trigger$useVariables, _trigger$useVariables2;
    const _useFlowContext = (0, _FlowContext.useFlowContext)(),
      workflow = _useFlowContext.workflow;
    const trigger = _triggers.triggers.get(workflow.type);
    return (_trigger$useVariables = trigger === null || trigger === void 0 ? void 0 : (_trigger$useVariables2 = trigger.useVariables) === null || _trigger$useVariables2 === void 0 ? void 0 : _trigger$useVariables2.call(trigger, workflow.config, options)) !== null && _trigger$useVariables !== void 0 ? _trigger$useVariables : null;
  }
};
exports.triggerOptions = triggerOptions;
const scopeOptions = {
  label: `{{t("Scope variables", { ns: "${_locale.NAMESPACE}" })}}`,
  value: '$scopes',
  useOptions(options) {
    const current = (0, _nodes.useNodeContext)();
    const scopes = (0, _nodes.useUpstreamScopes)(current);
    const result = [];
    scopes.forEach(node => {
      var _instruction$useScope;
      const instruction = _nodes.instructions.get(node.type);
      const subOptions = (_instruction$useScope = instruction.useScopeVariables) === null || _instruction$useScope === void 0 ? void 0 : _instruction$useScope.call(instruction, node, options);
      if (subOptions) {
        var _node$title2;
        result.push({
          key: node.id.toString(),
          value: node.id.toString(),
          label: (_node$title2 = node.title) !== null && _node$title2 !== void 0 ? _node$title2 : `#${node.id}`,
          children: subOptions
        });
      }
    });
    return result;
  }
};
exports.scopeOptions = scopeOptions;
const systemOptions = {
  label: `{{t("System variables", { ns: "${_locale.NAMESPACE}" })}}`,
  value: '$system',
  useOptions({
    types
  }) {
    return [...(!types || types.includes('date') ? [{
      key: 'now',
      value: 'now',
      label: `{{t("System time")}}`
    }] : [])];
  }
};
exports.systemOptions = systemOptions;
const BaseTypeSets = {
  boolean: new Set(['checkbox']),
  number: new Set(['number', 'percent']),
  string: new Set(['input', 'password', 'email', 'phone', 'select', 'radioGroup', 'text', 'markdown', 'richText', 'expression', 'time']),
  date: new Set(['date', 'createdAt', 'updatedAt'])
};
// { type: 'reference', options: { collection: 'users', multiple: false } }
// { type: 'reference', options: { collection: 'attachments', multiple: false } }
// { type: 'reference', options: { collection: 'myExpressions', entity: false } }
exports.BaseTypeSets = BaseTypeSets;
function matchFieldType(field, type, depth) {
  const inputType = typeof type;
  if (inputType === 'string') {
    var _BaseTypeSets$type;
    return (_BaseTypeSets$type = BaseTypeSets[type]) === null || _BaseTypeSets$type === void 0 ? void 0 : _BaseTypeSets$type.has(field.interface);
  }
  if (inputType === 'object' && type.type === 'reference') {
    if (isAssociationField(field)) {
      var _type$options, _type$options2, _type$options3;
      return ((_type$options = type.options) === null || _type$options === void 0 ? void 0 : _type$options.entity) && (field.collectionName === ((_type$options2 = type.options) === null || _type$options2 === void 0 ? void 0 : _type$options2.collection) || ((_type$options3 = type.options) === null || _type$options3 === void 0 ? void 0 : _type$options3.collection) === '*');
    } else if (field.isForeignKey) {
      var _type$options4, _type$options5;
      return field.collectionName === ((_type$options4 = type.options) === null || _type$options4 === void 0 ? void 0 : _type$options4.collection) && field.name === 'id' || field.target === ((_type$options5 = type.options) === null || _type$options5 === void 0 ? void 0 : _type$options5.collection);
    } else {
      return false;
    }
  }
  if (inputType === 'function') {
    return type(field, depth);
  }
  return false;
}
function isAssociationField(field) {
  return ['belongsTo', 'hasOne', 'hasMany', 'belongsToMany'].includes(field.type);
}
function filterTypedFields(fields, types, depth = 1) {
  if (!types) {
    return fields;
  }
  return fields.filter(field => {
    if (isAssociationField(field) && depth && filterTypedFields(useNormalizedFields(field.target), types, depth - 1).length) {
      return true;
    }
    return types.some(type => matchFieldType(field, type, depth));
  });
}
function useWorkflowVariableOptions(options = {}) {
  const compile = (0, _client().useCompile)();
  const result = [scopeOptions, nodesOptions, triggerOptions, systemOptions].map(item => {
    const opts = typeof item.useOptions === 'function' ? item.useOptions(options).filter(Boolean) : null;
    return {
      label: compile(item.label),
      value: item.value,
      key: item.value,
      children: compile(opts),
      disabled: opts && !opts.length
    };
  });
  return result;
}
function useNormalizedFields(collectionName) {
  const compile = (0, _client().useCompile)();
  const _useCollectionManager = (0, _client().useCollectionManager)(),
    getCollectionFields = _useCollectionManager.getCollectionFields;
  const fields = getCollectionFields(collectionName);
  const foreignKeyFields = [];
  const otherFields = [];
  fields.forEach(field => {
    if (field.isForeignKey) {
      foreignKeyFields.push(field);
    } else {
      otherFields.push(field);
    }
  });
  for (let i = otherFields.length - 1; i >= 0; i--) {
    const field = otherFields[i];
    if (field.type === 'belongsTo') {
      const foreignKeyField = foreignKeyFields.find(f => f.name === field.foreignKey);
      if (foreignKeyField) {
        var _field$uiSchema, _field$uiSchema2;
        otherFields.splice(i, 0, _objectSpread(_objectSpread(_objectSpread({}, field), foreignKeyField), {}, {
          uiSchema: _objectSpread(_objectSpread({}, field.uiSchema), {}, {
            title: ((_field$uiSchema = field.uiSchema) === null || _field$uiSchema === void 0 ? void 0 : _field$uiSchema.title) ? `${compile((_field$uiSchema2 = field.uiSchema) === null || _field$uiSchema2 === void 0 ? void 0 : _field$uiSchema2.title)} ID` : foreignKeyField.name
          })
        }));
      } else {
        var _field$uiSchema3, _field$uiSchema4;
        otherFields.splice(i, 0, _objectSpread(_objectSpread({}, field), {}, {
          name: field.foreignKey,
          type: 'bigInt',
          isForeignKey: true,
          interface: field.interface,
          uiSchema: _objectSpread(_objectSpread({}, field.uiSchema), {}, {
            title: ((_field$uiSchema3 = field.uiSchema) === null || _field$uiSchema3 === void 0 ? void 0 : _field$uiSchema3.title) ? `${compile((_field$uiSchema4 = field.uiSchema) === null || _field$uiSchema4 === void 0 ? void 0 : _field$uiSchema4.title)} ID` : field.name
          })
        }));
      }
    } else if (field.type === 'context' && field.collectionName === 'users') {
      var _otherFields$find, _belongsToField$uiSch, _belongsToField$uiSch2;
      const belongsToField = (_otherFields$find = otherFields.find(f => f.type === 'belongsTo' && f.target === 'users' && f.foreignKey === field.name)) !== null && _otherFields$find !== void 0 ? _otherFields$find : {};
      otherFields.splice(i, 0, _objectSpread(_objectSpread({}, field), {}, {
        type: field.dataType,
        interface: belongsToField.interface,
        uiSchema: _objectSpread(_objectSpread({}, belongsToField.uiSchema), {}, {
          title: ((_belongsToField$uiSch = belongsToField.uiSchema) === null || _belongsToField$uiSch === void 0 ? void 0 : _belongsToField$uiSch.title) ? `${compile((_belongsToField$uiSch2 = belongsToField.uiSchema) === null || _belongsToField$uiSch2 === void 0 ? void 0 : _belongsToField$uiSch2.title)} ID` : field.name
        })
      }));
    }
  }
  return otherFields.filter(field => field.interface && !field.hidden);
}
function useCollectionFieldOptions(options) {
  const fields = options.fields,
    collection = options.collection,
    types = options.types,
    _options$depth = options.depth,
    depth = _options$depth === void 0 ? 1 : _options$depth;
  const compile = (0, _client().useCompile)();
  const normalizedFields = useNormalizedFields(collection);
  const computedFields = fields !== null && fields !== void 0 ? fields : normalizedFields;
  const result = filterTypedFields(computedFields, types, depth).filter(field => !isAssociationField(field) || depth).map(field => {
    var _field$uiSchema5;
    const label = compile(((_field$uiSchema5 = field.uiSchema) === null || _field$uiSchema5 === void 0 ? void 0 : _field$uiSchema5.title) || field.name);
    return {
      label,
      key: field.name,
      value: field.name,
      children: isAssociationField(field) && depth ? useCollectionFieldOptions({
        collection: field.target,
        types,
        depth: depth - 1
      }) : null,
      field
    };
  });
  return result;
}