"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.PARALLEL_MODE = void 0;
var _constants = require("../constants");
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }
function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }
const PARALLEL_MODE = {
  ALL: 'all',
  ANY: 'any',
  RACE: 'race'
};
exports.PARALLEL_MODE = PARALLEL_MODE;
const Modes = {
  [PARALLEL_MODE.ALL]: {
    next(previous) {
      return previous.status >= _constants.JOB_STATUS.PENDING;
    },
    getStatus(result) {
      const failedStatus = result.find(status => status != null && status < _constants.JOB_STATUS.PENDING);
      if (typeof failedStatus !== 'undefined') {
        return failedStatus;
      }
      if (result.every(status => status != null && status === _constants.JOB_STATUS.RESOLVED)) {
        return _constants.JOB_STATUS.RESOLVED;
      }
      return _constants.JOB_STATUS.PENDING;
    }
  },
  [PARALLEL_MODE.ANY]: {
    next(previous) {
      return previous.status <= _constants.JOB_STATUS.PENDING;
    },
    getStatus(result) {
      if (result.some(status => status != null && status === _constants.JOB_STATUS.RESOLVED)) {
        return _constants.JOB_STATUS.RESOLVED;
      }
      if (result.some(status => status != null ? status === _constants.JOB_STATUS.PENDING : true)) {
        return _constants.JOB_STATUS.PENDING;
      }
      return _constants.JOB_STATUS.FAILED;
    }
  },
  [PARALLEL_MODE.RACE]: {
    next(previous) {
      return previous.status === _constants.JOB_STATUS.PENDING;
    },
    getStatus(result) {
      if (result.some(status => status != null && status === _constants.JOB_STATUS.RESOLVED)) {
        return _constants.JOB_STATUS.RESOLVED;
      }
      const failedStatus = result.find(status => status != null && status < _constants.JOB_STATUS.PENDING);
      if (typeof failedStatus !== 'undefined') {
        return failedStatus;
      }
      return _constants.JOB_STATUS.PENDING;
    }
  }
};
var _default = {
  run(node, prevJob, processor) {
    return _asyncToGenerator(function* () {
      var _prevJob$id;
      const branches = processor.getBranches(node);
      const job = yield processor.saveJob({
        status: _constants.JOB_STATUS.PENDING,
        result: Array(branches.length).fill(null),
        nodeId: node.id,
        upstreamId: (_prevJob$id = prevJob === null || prevJob === void 0 ? void 0 : prevJob.id) !== null && _prevJob$id !== void 0 ? _prevJob$id : null
      });
      // NOTE:
      // use `reduce` but not `Promise.all` here to avoid racing manupulating db.
      // for users, this is almost equivalent to `Promise.all`,
      // because of the delay is not significant sensible.
      // another benifit of this is, it could handle sequenced branches in future.
      const _node$config$mode = node.config.mode,
        mode = _node$config$mode === void 0 ? PARALLEL_MODE.ALL : _node$config$mode;
      yield branches.reduce((promise, branch, i) => promise.then( /*#__PURE__*/function () {
        var _ref = _asyncToGenerator(function* (previous) {
          if (i && !Modes[mode].next(previous)) {
            return previous;
          }
          yield processor.run(branch, job);
          // find last job of the branch
          return processor.findBranchLastJob(branch);
        });
        return function (_x) {
          return _ref.apply(this, arguments);
        };
      }()), Promise.resolve());
      return null;
    })();
  },
  resume(node, branchJob, processor) {
    return _asyncToGenerator(function* () {
      const job = processor.findBranchParentJob(branchJob, node);
      const result = job.result,
        status = job.status;
      // if parallel has been done (resolved / rejected), do not care newly executed branch jobs.
      if (status !== _constants.JOB_STATUS.PENDING) {
        return processor.exit();
      }
      // find the index of the node which start the branch
      const jobNode = processor.nodesMap.get(branchJob.nodeId);
      const branchStartNode = processor.findBranchStartNode(jobNode, node);
      const branches = processor.getBranches(node);
      const branchIndex = branches.indexOf(branchStartNode);
      const _ref2 = node.config || {},
        _ref2$mode = _ref2.mode,
        mode = _ref2$mode === void 0 ? PARALLEL_MODE.ALL : _ref2$mode;
      const newResult = [...result.slice(0, branchIndex), branchJob.status, ...result.slice(branchIndex + 1)];
      job.set({
        result: newResult,
        status: Modes[mode].getStatus(newResult)
      });
      if (job.status === _constants.JOB_STATUS.PENDING) {
        yield job.save({
          transaction: processor.transaction
        });
        return processor.exit();
      }
      return job;
    })();
  }
};
exports.default = _default;